{% extends 'generic/object.html' %}
{% load helpers %}
{% load render_table from django_tables2 %}

{% block content %}
    <div class="row mb-3">
        <div class="col col-md-12">
            <div class="card">
                <h5 class="card-header">Backups for {{ object.name }}</h5>
                <div class="card-body">
                    {% render_table table 'inc/table.html' %}
                </div>
            </div>
        </div>
    </div>
{% endblock %}