# Generated by Django 5.2.12 on 2026-03-28 14:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_proxbox', '0010_alter_fastapiendpoint_options_and_more'),
        ('netbox_proxbox', '0010_netboxendpoint_token_v2_support'),
    ]

    operations = [
    ]
