# Generated by Django 5.2.12 on 2026-03-28 16:30

from django.db import migrations

import netbox_proxbox.fields


class Migration(migrations.Migration):
    dependencies = [
        ("netbox_proxbox", "0012_merge_0011_merge_0010_heads_0011_merge_20260328_1454"),
    ]

    operations = [
        migrations.AlterField(
            model_name="fastapiendpoint",
            name="domain",
            field=netbox_proxbox.fields.DomainField(
                blank=True,
                help_text="Domain name of the ProxBox backend service.",
                max_length=253,
                null=True,
                verbose_name="Domain",
            ),
        ),
        migrations.AlterField(
            model_name="netboxendpoint",
            name="domain",
            field=netbox_proxbox.fields.DomainField(
                blank=True,
                help_text="Domain name of the remote NetBox API.",
                max_length=253,
                null=True,
                verbose_name="Domain",
            ),
        ),
    ]
