<div id="virtual-machines-div" class="form form-horizontal">
    <h2 id="proxmox-virtual-machines-table" class="text-center">Virtual Machines</h2>
    <h3>Synced Virtual Machines: <span id='synced-vm-count'>0</span> / <span id='total-vm-count'>0</span> (<span id='sync-percentage-ratio'>0</span>)</h3>
    <div class="card">
        <div class="table-responsive" id="object_list">
            <div class="table-responsive">
                <table class="table table-hover object-list">
                    <thead>
                        <tr>
                            <th>Status</th>
                            <th>NetBox ID</th>
                            <th>Name</th>
                            <th>VM/CT Status</th>
                            <th>Device</th>
                            <th>Cluster</th>
                            <th>VM Interfaces</th>
                            <th>Role</th>
                            <th>Virtual CPUs</th>
                            <th>Memory (MB)</th>
                            <th>Disk Space (GB)</th>
                            <th>IP Address</th>
                            <th></th>
                        </tr>
                    </thead>
                    <tbody id="virtual-machine-table-data">
                        <!-- Show it if no virtual machines synced. -->
                        <tr id='virtual-machines-table-default-td'>
                            <td colspan="12" class="text-center text-muted">— No virtual machines could be synced with Proxmox —</td>
                        </tr>
                    </tbody>
                </table>
            </div>
        </div>
    </div>
</div>
