# Generated by Django 5.2.12 on 2026-03-28 15:05

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_proxbox', '0011_merge_0010_heads'),
        ('netbox_proxbox', '0011_merge_20260328_1454'),
    ]

    operations = [
    ]
